import os
import requests
from libbananadm import vault
from libbananadm import policies
from hvac import exceptions


def init(args):
    if args.from_scratch:
        root_token, keys, token = init_from_scratch(args)

    client = vault.get_vault_client(args)

    try:
        client.sys.enable_auth_method(
            method_type='cert',
            path='{}/cert'.format(args.root_path),
        )
        print(
            'enabled cert auth method on path /auth/{}/cert'
            .format(args.root_path)
        )
    except exceptions.InvalidRequest as error:
        if 'path is already in use' in error.errors[0]:
            print(
                'error: looks like this vault was already setup for banana, '
                + 'aborting'
            )
            exit(1)
        raise error

    monitor_policy_name = 'banana-monitor'
    client.sys.create_or_update_policy(
        name=monitor_policy_name,
        policy=policies.generate_monitor_policy(args),
    )
    print('created policy {}'.format(monitor_policy_name))

    if args.from_scratch:
        print(
            '\n==========================\n'
            'Your Vault instance is now ready to go.\n'
            'To be able to reach it, '
            'you should add this to your shell rc file :\n\n'
            'export VAULT_ADDR={}\n'
            'export VAULT_TOKEN={}\n\n'
            'Here is your Vault unseal keys, you should store them safely:\n\n'
            '{}\n\n'
            'Your Vault root token is: {} (but you shouldn\'t need it)'
            .format(
                os.getenv('VAULT_ADDR'),
                token,
                '\n'.join(keys),
                root_token,
            )
        )


def init_from_scratch(args):
    client = vault.get_vault_client(args)

    # TODO: Use more shares/threshold for security
    res = client.sys.initialize(1, 1)
    root_token = res['root_token']
    keys = res['keys']
    print('initialized vault')

    client.sys.submit_unseal_keys(keys)
    print('unsealed vault')

    os.environ['VAULT_TOKEN'] = root_token
    client = vault.get_vault_client(args)

    res = requests.get(
        'https://raw.githubusercontent.com/enix/banana/master/'
        'config/vault/bananadm-policy.hcl',
    )
    if res.status_code >= 400:
        print(res.json())
        return
    policy = res.text
    print('downloaded bananadm policy from github')

    bananadm_policy_name = 'bananadm'
    client.sys.create_or_update_policy(
        name=bananadm_policy_name,
        policy=policy,
    )
    print('uploaded policy {}'.format(bananadm_policy_name))

    token_res = client.create_token(policies=['bananadm'])
    token = token_res['auth']['client_token']

    os.environ['VAULT_TOKEN'] = token
    return root_token, keys, token


def reconfigure(args):
    client = vault.get_vault_client(args)
    token = client.create_token(
        policies=['banana-monitor'],
        lease='1h',
    )

    res = requests.post(
        args.monitor_addr + '/reconfigure',
        data='{}\n{}'.format(
            os.getenv('VAULT_ADDR'),
            token['auth']['client_token'],
        ),
        verify=not args.tls_skip_verify,
    )
    if res.status_code >= 400:
        print(res.json())

    print('successfully reloaded nginx trust configuration')



import os
import sys
from libbananadm import vault
from tabulate import tabulate


def create_user(args):
    client_pki = '{}/{}/users-pki'.format(args.root_path, args.client)
    sys.stderr.write(
        'issuing certificate with CN \'{}\' using PKI {}\n'
        .format(args.name, client_pki)
    )
    cert, key = vault.create_cert(
        args, args.name, client_pki, 'default'
    )
    open(args.name + '.pem', 'w').write(cert)
    open(args.name + '.key', 'w').write(key)

    sys.stderr.write('creating p12 file\n')
    os.system(
        'openssl pkcs12 -export -inkey {}.key -in {}.pem'
        .format(args.name, args.name, args.name)
    )
    os.system('rm {}.pem'.format(args.name))
    os.system('rm {}.key'.format(args.name))

    sys.stderr.write('successfully wrote p12 data to stdout\n')


def list_users(args):
    mount_point = '{}/{}/users-pki'.format(args.root_path, args.client)
    users = filter(
        lambda x: x[0] != '{} User Intermediate CA'.format(args.client),
        vault.list_cn_from_pki(args, mount_point),
    )

    print(tabulate(users, headers=['Name', 'Serial']))



agent_install_policy_template = '''
path "{root_path}/{company}/agents-pki/issue/default" {{
    capabilities = ["create", "update"]
}}
'''

agent_access_policy_template = '''
path "{root_path}/{company}/secrets/data/backends/*" {{
    capabilities = ["read"]
}}

path "{root_path}/{company}/secrets/data/agents/{{{{identity.entity.id}}}}"\
{{
    capabilities = ["read", "list", "create", "update"]
}}
'''

monitor_policy_template = '''
path "sys/mounts" {{
  capabilities = ["read"]
}}

path "{root_path}/+/root-pki/ca/pem" {{
  capabilities = ["read"]
}}

path "{root_path}/+/agents-pki/ca/pem" {{
  capabilities = ["read"]
}}

path "{root_path}/+/users-pki/ca/pem" {{
  capabilities = ["read"]
}}
'''


def generate_agent_install_policy(args):
    return agent_install_policy_template.format(
        root_path=args.root_path,
        company=args.name,
    )


def generate_agent_access_policy(args):
    return agent_access_policy_template.format(
        root_path=args.root_path,
        company=args.name,
    )


def generate_monitor_policy(args):
    return monitor_policy_template.format(
        root_path=args.root_path,
    )
